#ifndef __AUTOTILES_H__
#define __AUTOTILES_H__

#include <string>
#include <vector>
#include <fstream>
#include <iostream>
#include "tiles.h"
#include "tm.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
//  Turing Machine assumptions:
//    - tape is one-way infinite to the right
//    - the TM will begin in the start state with the tape head on the leftmost cell of the tape
//    - the tape alphabet is exactly the input alphabet plus the blank symbol '_'
//
//  Naming conventions:
//    - if a name is not explicitly set for a tile type, it will be automatically generated in
//      the following format:
//        <north label>|<east lable>|<south label>|<west label>
//
///////////////////////////////////////////////////////////////////////////////////////////////////



//  NOTE:  When adding to this list of reserved characters, make sure to add to the ReservedChars array too!
#define EMPTY       "."
#define COPYLEFT    "<"
#define COPYRIGHT   ">"
#define NAMEDELIMIT "|"
char* ReservedChars[] = {BLANK, EMPTY, COPYLEFT, COPYRIGHT, NAMEDELIMIT };

#define STATES_STR      "States:"
#define ALPHABET_STR    "Alphabet:"
#define START_STR       "Start state:"
#define HALTING_STR     "Halting state:"
#define ACCEPTING_STR   "Accepting state:"
#define REJECTING_STR   "Rejecting state:"
#define TRANSITION_STR  "Transitions:"

#define ACCEPT    "ACCEPT"
#define REJECT    "REJECT"

//  The set of possible counter values to pass through:
std::string ctrVals[] = {"M0*L", "M1*L", "M1", "M1*", "0*L", "1L", "0", "0*", "1", "_", "_*", ""};

//  The possible values in the most significant bit position
std::string MostSigArray[] = {"M0*L", "M1*L", "M1", "M1*", ""};

//  The counter values not in MSB position
std::string NonMostSigArray[] = { "0*L", "1L", "0", "0*", "1", ""};

int GetCounterBit(std::string cv);
std::string WrapCounterValue(std::string cv) { return std::string("~(" + cv + ")"); };
std::string FormatPassthroughValue(const char* pv) { return std::string(std::string("(") + pv + ")"); };

void PrintTemplate(const char* tmplFileName);

#endif //  __AUTOTILES_H__
